//PROGRAMM FOR BUTTONs, BUZZER, RELAY AND LED
//
// PROCESSOR : PIC16F876A
// CLOCK	 : 20MHz, EXTERNAL


#include	<pic.h>

__CONFIG(UNPROTECT & PWRTEN & HS & BORDIS & WDTDIS & LVPDIS);
  
#define bitset(var,bitno) ((var) |= 1 << (bitno))
#define bitclr(var,bitno) ((var) &= ~(1 << (bitno)))


#define	COL1		RB0	
#define	COL2		RB1	
#define	COL3		RB2	
#define	LED			RB4
#define	BUZZ1		RC4			
#define	BUZZ2		RC5	
#define	RELAY		RA1

#define	OFF		0
#define	ON		1

#define BUTTON_NONE			0
#define BUTTON1_ON			1
#define BUTTON1_OFF			2
#define BUTTON2_ON			3
#define BUTTON2_OFF			4
#define BUTTON3_ON			5
#define BUTTON3_OFF			6
#define BUTTON4_ON			7
#define BUTTON4_OFF			8
#define BUTTON5_ON			9
#define BUTTON5_OFF			10
#define BUTTON6_ON			11
#define BUTTON6_OFF			12

	

unsigned char B1, B2, B3, B4, B5, B6;


#define		DELAY_1000us			498		//498*2+3 = 999us
#define		DELAY_125us				61		//61*2+3 = 125us 
#define		DELAY_250us				123		//123*2+3 = 249us 

unsigned  int dly;
unsigned  int i;
unsigned  char but;

//delay time = (cntr*2)+3 (in us for 20MHz)
//FULL OPTIMIZATION of C compilator(10 level, enable post-pass optimisation)
void Delay(unsigned int cntr) {									
	for (dly=0; dly!=cntr; dly++) {
		asm("nop");
		asm("nop");	
		asm("nop");	
	}
}

//delay time is ~ 1ms	
void DelayMs (unsigned char e)		
{
	while(--e) {
		Delay(DELAY_1000us);
	 }
}

char ButtonScan(void) {

	//set COL1 ---------------------------------------------
	PORTB |= 0x07;
	TRISB |= 0x07;
	bitclr(TRISB, 0);	 						
	DelayMs(2);
	
	//button 1 is pressed		
	if((COL2==1)&&(B1==OFF)) { B1=ON; return BUTTON1_ON; }
	if((COL2==0)&&(B1==ON))  { B1=OFF; return BUTTON1_OFF; }
	
	//button 6 is pressed
	if((COL3==1)&&(B6==OFF)) { B6=ON; return BUTTON6_ON; }	
	if((COL3==0)&&(B6==ON))  { B6=OFF; return BUTTON6_OFF; }
	
	//set COL1 ---------------------------------------------
	PORTB |= 0x07;
	TRISB |= 0x07;
	bitclr(TRISB,1);								
	DelayMs(2);
	
	//button 4 is pressed
	if((COL1==1)&&(B4==OFF)) { B4=ON; return BUTTON4_ON; }
	if((COL1==0)&&(B4==ON))  { B4=OFF; return BUTTON4_OFF; }			
	
	//button 2 is pressed
	if((COL3==1)&&(B2==OFF)) { B2=ON; return BUTTON2_ON; }
	if((COL3==0)&&(B2==ON))  { B2=OFF; return BUTTON2_OFF; }
	
	//set COL1 ---------------------------------------------
	PORTB |= 0x07;
	TRISB |= 0x03;
	bitclr(TRISB,2);							
	DelayMs(2);
	
	//button 3 is pressed
	if((COL1==1)&&(B3==OFF)) { B3=ON; return BUTTON3_ON; }
	if ((COL1==0)&&(B3==ON)) { B3=OFF; return BUTTON3_OFF; }
	
	//button 5 is pressed
	if((COL2==1)&&(B5==OFF)) { B5=ON; return BUTTON5_ON; }
	if((COL2==0)&&(B5==ON))  { B5=OFF; return BUTTON5_OFF; }				

	return BUTTON_NONE;	
}

void Beep1(void) {

	unsigned char t=150;

	while(--t) {
		BUZZ2 = OFF;
		BUZZ1 = ON;					
		Delay(DELAY_125us);				
		BUZZ1 = OFF;
		BUZZ2 = ON;
	 	Delay(DELAY_125us);
	}
}

void Beep2(void) {

	unsigned char t=75;

	while(--t) {
		BUZZ2 = OFF;
		BUZZ1 = ON;					
		Delay(DELAY_250us);				
		BUZZ1 = OFF;
		BUZZ2 = ON;
	 	Delay(DELAY_250us);
	}
}

// main function
void main( void ) {

	// PORTA
	INTCON 	= 0x0;			// Disable inerupt
	CMCON 	= 0x07;		    // Comparators Off
	ADCON1  = 0x06;			// Port as Digital IO
	TRISA1 	= 0;			// RELAY pin as output

	// PORTB
	INTCON 	= 0x00;		// Disable inerupt
	//OPTION 	= 0x80;		// PORTB pull-ups are disabled
	TRISB 	= 0xEF;     // All as input, except LED pin
	LED = ON;

	// PORTC
	INTCON 	= 0x00;		// Disable inerupt
	TRISC 	= 0xCF;     // All as input, except BUZZER pins


	// Clear variable
	B1=B2=B3=B4=B5=B6=0;

	// loop forever - echo 
	while(1) {

		// Scan Button
	 	but = ButtonScan();
	
		if(but==BUTTON1_ON) 		{ LED = OFF; /* your code here */ }
		else if(but==BUTTON1_OFF) 	{ LED = ON; /* your code here */ }
		else if(but==BUTTON2_ON)  	{ RELAY = ON; /* your code here */ }
		else if(but==BUTTON2_OFF) 	{ RELAY = OFF; /* your code here */ }
		else if(but==BUTTON3_ON)  	{ Beep1(); /* your code here */ }
		else if(but==BUTTON3_OFF) 	{ Beep2(); /* your code here */ }
		else if(but==BUTTON4_ON)  	{ Beep1(); /* your code here */ }
		else if(but==BUTTON4_OFF) 	{ Beep2(); /* your code here */ }
		else if(but==BUTTON5_ON)  	{ Beep1(); /* your code here */ }
		else if(but==BUTTON5_OFF) 	{ Beep2(); /* your code here */ }
		else if(but==BUTTON6_ON)  	{ Beep1(); /* your code here */ }
		else if(but==BUTTON6_OFF) 	{ Beep2(); /* your code here */ }

	}
}
